@@ -126,9 +126,6 @@ static void
 refresh_cb (GtkWidget *w, gpointer data)
 {
 	g_return_if_fail(GNOME_DB_IS_GRID(data));
-  
-	/* FIXME: must pass GnomeDbGrid widget pointer */
-	gnome_db_grid_refresh(GNOME_DB_GRID(data));
 }
 
 static void
@@ -293,44 +290,50 @@ create_empty_grid (GnomeDbGrid *grid)
 	grid->total_rows = 0;
 }
 
+#define ROWS_TO_FETCH 100
+
 static gint
 timeout_callback (GnomeDbGrid *grid)
 {
-	gint position, cnt;
+	gint position, cnt, rows;
 
 	g_return_val_if_fail(GNOME_DB_IS_GRID(grid), 0);
 	g_return_val_if_fail(GTK_IS_CLIST(grid->grid), 0);
 	g_return_val_if_fail(GDA_IS_RECORDSET(grid->recset), 0);
 
-	position = gda_recordset_move(grid->recset, 1, 0);
-	if (position != GDA_RECORDSET_INVALID_POSITION &&
-	    !gda_recordset_eof(grid->recset)) {
-		gchar *str;
-		GPtrArray *row = g_ptr_array_new();
+	gtk_clist_freeze(GTK_CLIST(grid->grid));
+	for (rows = 0; rows < ROWS_TO_FETCH; rows++) {
+		position = gda_recordset_move(grid->recset, 1, 0);
+		if (position != GDA_RECORDSET_INVALID_POSITION &&
+		    !gda_recordset_eof(grid->recset)) {
+			gchar *str;
+			GPtrArray *row = g_ptr_array_new();
 
-		gtk_clist_freeze(GTK_CLIST(grid->grid));
-		for (cnt = 0; cnt < gda_recordset_rowsize(grid->recset); cnt++) {
-			GdaField *field = gda_recordset_field_idx(grid->recset, cnt);
-			g_ptr_array_add(row, (gpointer) gda_stringify_value(0, 0, field));
+			for (cnt = 0; cnt < gda_recordset_rowsize(grid->recset); cnt++) {
+				GdaField *field = gda_recordset_field_idx(grid->recset, cnt);
+				g_ptr_array_add(row, (gpointer) gda_stringify_value(0, 0, field));
+			}
+			gtk_clist_append(GTK_CLIST(grid->grid), (gchar **) row->pdata);
+			gtk_signal_emit(GTK_OBJECT(grid),
+					gnome_db_grid_signals[GNOME_DB_GRID_ADD_ROW],
+					grid->total_rows);
+			g_ptr_array_free(row, TRUE);
+
+			/* move to next row */
+			str = g_strdup_printf("%d", grid->total_rows + 1);
+			g_free((gpointer) str);
+			grid->total_rows++;
 		}
-		gtk_clist_append(GTK_CLIST(grid->grid), (gchar **) row->pdata);
-		gtk_signal_emit(GTK_OBJECT(grid),
-				gnome_db_grid_signals[GNOME_DB_GRID_ADD_ROW],
-				grid->total_rows);
-		gtk_clist_thaw(GTK_CLIST(grid->grid));
-		g_ptr_array_free(row, TRUE);
+		else {
+			gtk_clist_thaw (GTK_CLIST (grid->grid));
+			gtk_signal_emit_by_name (GTK_OBJECT (grid), "loaded");
+			grid->timeout_handle = -1;
+			return 0;
+		}
+	}
+	gtk_clist_thaw (GTK_CLIST (grid->grid));
 
-		/* move to next row */
-		str = g_strdup_printf("%d", grid->total_rows + 1);
-		g_free((gpointer) str);
-		grid->total_rows++;
-	}
-	else {
-		gtk_signal_emit_by_name(GTK_OBJECT(grid), "loaded");
-		grid->timeout_handle = -1;
-		return (0);
-	}
-	return (1);
+	return 1;
 }
 
 static void
@@ -359,7 +362,7 @@ show_recordset (GnomeDbGrid *grid, GdaRecordset *recset)
 		if (total_fields > 0) {
 			for (cnt = 0; cnt < total_fields; cnt++) {
 				gchar *col_name;
-				col_name = gda_field_name(gda_recordset_field_idx(recset, cnt));
+				col_name = gda_field_get_name(gda_recordset_field_idx(recset, cnt));
 				if (col_name != NULL)
 					g_ptr_array_add(columns, (gpointer) col_name);
 			}
@@ -368,7 +371,7 @@ show_recordset (GnomeDbGrid *grid, GdaRecordset *recset)
 
 			/* column justification */
 			for (cnt = 0; cnt < total_fields; cnt++) {
-				switch (gda_field_type(gda_recordset_field_idx(recset, cnt))) {
+				switch (gda_field_get_gdatype(gda_recordset_field_idx(recset, cnt))) {
 				case GDA_TypeBigint :
 				case GDA_TypeCurrency :
 				case GDA_TypeDecimal :
